package xyz.lychee.lagfixer.libs.adventure.platform.bukkit;

import java.lang.invoke.MethodHandle;
import java.lang.reflect.Method;
import net.md_5.bungee.api.chat.BaseComponent;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.Facet;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.FacetBase;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.Knob;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.gson.GsonComponentSerializer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

class PaperFacet<V extends CommandSender> extends FacetBase<V> {
   private static final boolean SUPPORTED = Knob.isEnabled("paper", true);
   static final Class<?> NATIVE_COMPONENT_CLASS = MinecraftReflection.findClass(String.join(".", "net", "kyori", "adventure", "text", "Component"));
   private static final Class<?> NATIVE_GSON_COMPONENT_SERIALIZER_CLASS = MinecraftReflection.findClass(String.join(".", "net", "kyori", "adventure", "text", "serializer", "gson", "GsonComponentSerializer"));
   private static final Class<?> NATIVE_GSON_COMPONENT_SERIALIZER_IMPL_CLASS = MinecraftReflection.findClass(String.join(".", "net", "kyori", "adventure", "text", "serializer", "gson", "GsonComponentSerializerImpl"));
   private static final MethodHandle NATIVE_GSON_COMPONENT_SERIALIZER_GSON_GETTER;
   private static final MethodHandle NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD;

   private static @Nullable MethodHandle findNativeDeserializeMethod() {
      try {
         Method method = NATIVE_GSON_COMPONENT_SERIALIZER_IMPL_CLASS.getDeclaredMethod("deserialize", String.class);
         method.setAccessible(true);
         return MinecraftReflection.lookup().unreflect(method);
      } catch (IllegalAccessException | NullPointerException | NoSuchMethodException var1) {
         return null;
      }
   }

   protected PaperFacet(final @Nullable Class<? extends V> viewerClass) {
      super(viewerClass);
   }

   public boolean isSupported() {
      return super.isSupported() && SUPPORTED;
   }

   static {
      NATIVE_GSON_COMPONENT_SERIALIZER_GSON_GETTER = MinecraftReflection.findStaticMethod(NATIVE_GSON_COMPONENT_SERIALIZER_CLASS, "gson", NATIVE_GSON_COMPONENT_SERIALIZER_CLASS);
      NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD = findNativeDeserializeMethod();
   }

   static class TabList extends CraftBukkitFacet.TabList {
      private static final boolean SUPPORTED;
      private static final MethodHandle NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD_BOUND;

      private static @Nullable MethodHandle createBoundNativeDeserializeMethodHandle() {
         if (SUPPORTED) {
            try {
               return PaperFacet.NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD.bindTo(PaperFacet.NATIVE_GSON_COMPONENT_SERIALIZER_GSON_GETTER.invoke());
            } catch (Throwable var1) {
               Throwable throwable = var1;
               Knob.logError(throwable, "Failed to access native GsonComponentSerializer", new Object[0]);
               return null;
            }
         } else {
            return null;
         }
      }

      public boolean isSupported() {
         return SUPPORTED && super.isSupported() && CLIENTBOUND_TAB_LIST_PACKET_SET_HEADER != null && CLIENTBOUND_TAB_LIST_PACKET_SET_FOOTER != null;
      }

      protected Object create117Packet(final Player viewer, final @Nullable Object header, final @Nullable Object footer) throws Throwable {
         Object packet = CLIENTBOUND_TAB_LIST_PACKET_CTOR.invoke((Void)null, (Void)null);
         CLIENTBOUND_TAB_LIST_PACKET_SET_HEADER.invoke(packet, header == null ? this.createMessage((Player)viewer, Component.empty()) : header);
         CLIENTBOUND_TAB_LIST_PACKET_SET_FOOTER.invoke(packet, footer == null ? this.createMessage((Player)viewer, Component.empty()) : footer);
         return packet;
      }

      public @Nullable Object createMessage(final @NotNull Player viewer, final @NotNull Component message) {
         try {
            return NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD_BOUND.invoke((String)GsonComponentSerializer.gson().serialize(message));
         } catch (Throwable var4) {
            Throwable throwable = var4;
            Knob.logError(throwable, "Failed to create native Component message", new Object[0]);
            return null;
         }
      }

      static {
         SUPPORTED = MinecraftReflection.hasField(CLASS_CRAFT_PLAYER, PaperFacet.NATIVE_COMPONENT_CLASS, "playerListHeader") && MinecraftReflection.hasField(CLASS_CRAFT_PLAYER, PaperFacet.NATIVE_COMPONENT_CLASS, "playerListFooter");
         NATIVE_GSON_COMPONENT_SERIALIZER_DESERIALIZE_METHOD_BOUND = createBoundNativeDeserializeMethodHandle();
      }
   }

   static class Title extends SpigotFacet.Message<Player> implements Facet.Title<Player, BaseComponent[], com.destroystokyo.paper.Title.Builder, com.destroystokyo.paper.Title> {
      private static final boolean SUPPORTED = MinecraftReflection.hasClass("com.destroystokyo.paper.Title");

      protected Title() {
         super(Player.class);
      }

      public boolean isSupported() {
         return super.isSupported() && SUPPORTED;
      }

      public com.destroystokyo.paper.Title.@NotNull Builder createTitleCollection() {
         return com.destroystokyo.paper.Title.builder();
      }

      public void contributeTitle(final com.destroystokyo.paper.Title.@NotNull Builder coll, final BaseComponent @NotNull [] title) {
         coll.title(title);
      }

      public void contributeSubtitle(final com.destroystokyo.paper.Title.@NotNull Builder coll, final BaseComponent @NotNull [] subtitle) {
         coll.subtitle(subtitle);
      }

      public void contributeTimes(final com.destroystokyo.paper.Title.@NotNull Builder coll, final int inTicks, final int stayTicks, final int outTicks) {
         if (inTicks > -1) {
            coll.fadeIn(inTicks);
         }

         if (stayTicks > -1) {
            coll.stay(stayTicks);
         }

         if (outTicks > -1) {
            coll.fadeOut(outTicks);
         }

      }

      @Nullable
      public com.destroystokyo.paper.@Nullable Title completeTitle(final com.destroystokyo.paper.Title.@NotNull Builder coll) {
         return coll.build();
      }

      public void showTitle(final @NotNull Player viewer, final com.destroystokyo.paper.@NotNull Title title) {
         viewer.sendTitle(title);
      }

      public void clearTitle(final @NotNull Player viewer) {
         viewer.hideTitle();
      }

      public void resetTitle(final @NotNull Player viewer) {
         viewer.resetTitle();
      }
   }
}
